const objectHash = require('object-hash');

const verifySignature = require('../verify-signature');
const { forum } = require('../db');
const read = require('./read');

const write = async ({
  sender, authors, content, references, embeddedData, signature,
}) => {
  // Check author signature
  if (!verifySignature({
    sender, authors, content, signature, embeddedData,
  })) {
    const err = new Error();
    err.status = 403;
    err.message = 'Signature verification failed';
    throw err;
  }

  // Compute content hash
  const data = {
    sender, authors, content, signature, embeddedData, references,
  };
  // We omit references from the hash in order to support forum graph import.
  // When a post is imported, the hashes can be precomputed for cited posts,
  // without traversing the graph infinitely to compute hashes along entire reference chain.
  const hash = objectHash({
    sender, authors, content, signature, embeddedData,
  });

  // Make sure a post with this hash has not already been written
  let existingPost;
  try {
    existingPost = await read(hash);
    // If this doesn't throw, it means a post with this hash was already written
  } catch (e) {
    if (e.status !== 404) {
      throw e;
    }
  }

  if (existingPost) {
    const err = new Error();
    err.status = 403;
    err.message = `A post with hash ${hash} already exists`;
    throw err;
  }

  // Store content
  await forum.put(hash, data);

  // Return hash
  return { hash, data };
};

module.exports = write;
